package hr.genetics.models;

import java.util.ArrayList;

public class Vehicle extends GenericVechileModel {

	/**
	 * Number of sensors
	 */
	private static int sensorsQuantity = 3;
	
	/**
	 * Resolution of sensors
	 */
	private static int sensorResolution = 8;
	
	private static int sensorFrequency = 20;
	
	/**
	 * Maximum angle that vehicle sees with sensors
	 */
	private static double maxAngle = 2*Math.PI/3; // 120 degrees
	
	private static double steerAngle = Math.PI/8;
	
	private double[] distances;
	private int[] diffDistance;
	private int[] choose;
	private int sensorClock;
		
	public Vehicle() {
		
		super(new int[]{sensorResolution*2+1, sensorResolution*2+1, sensorResolution*2+1});
						
	}
	
	/**
	 * Initialises sensors from -maxAngle to maxAngle
	 */
	public void initialiseSensors() {
		
		this.sensors = new ArrayList<Sensor>();
		
		this.sensorClock = 10;
		
		this.distances = new double[sensorsQuantity];
		this.diffDistance = new int[sensorsQuantity];
		this.choose = new int[sensorsQuantity];
		
		double angle = 0;
		for (int i = 0; i < sensorsQuantity; ++i) {
			angle = -maxAngle/2 + i*maxAngle/(sensorsQuantity-1);
			this.sensors.add(new Sensor(this.reality, angle));
		}
		
	}
	
	public void checkSensors() {
	
		double currDistance = 0;
		
		int size = this.sensors.size();
		for (int i = 0; i < size; ++i) {
			currDistance = this.sensors.get(i).getValue();
			this.diffDistance[i] = new Double(Math.round(currDistance - this.distances[i])).intValue();
			distances[i] = currDistance;
		}
		
	}

	
	@Override
	protected void calculateNewSpeed() {

		if (this.sensorClock != 0) {
			this.sensorClock--;
			return;
		} else {
			this.sensorClock = sensorFrequency;
		}
		
		// find value that will be represented as +-(sensorResolution)
		double maxAbsDifference = 0;
		for (int i = 0; i < this.diffDistance.length; ++i) {
			if (Math.abs(this.diffDistance[i]) > maxAbsDifference)
				maxAbsDifference = Math.abs(this.diffDistance[i]);
		}
				
		double coefficient = 0;
		if (maxAbsDifference != 0) {
			coefficient = Math.abs(sensorResolution/maxAbsDifference);
		}
				
		for (int i = 0; i < this.choose.length; ++i) {
			this.choose[i] = sensorResolution + 
				new Double(Math.round(this.diffDistance[i]*coefficient)).intValue();
		}
		
				
		Action chosenAction = this.getChromosome(this.choose);

		switch (chosenAction) {
			case CONTINUE:
					
			break;
			case LEFT:
				this.getSpeed().rotate(steerAngle);		
			break;
			case RIGHT:
				this.getSpeed().rotate(-1*steerAngle);
			break;
		}		
		
	}
	
}
